=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2009-02-18 01:00:02 +0000
+++ loggerhead/controllers/revision_ui.py	2009-03-05 23:41:51 +0000
@@ -20,6 +20,7 @@
 from StringIO import StringIO
 
 import bzrlib.diff
+from bzrlib import errors
 
 from paste.httpexceptions import HTTPServerError
 
@@ -80,7 +81,7 @@
             chunks.append(chunk)
         return chunks
 
-    def _parse_diffs(self, old_tree, new_tree, delta):
+    def _parse_diffs(self, old_tree, new_tree, delta, specific_path):
         """
         Return a list of processed diffs, in the format::
 
@@ -100,16 +101,40 @@
         process = []
         out = []
 
+        def include_specific_path(path):
+            return specific_path == path
+        def include_all_paths(path):
+            return True
+        if specific_path:
+            include_path = include_specific_path
+        else:
+            include_path = include_all_paths
+
         for old_path, new_path, fid, \
             kind, text_modified, meta_modified in delta.renamed:
-            if text_modified:
+            if text_modified and include_path(new_path):
                 process.append((old_path, new_path, fid, kind))
         for path, fid, kind, text_modified, meta_modified in delta.modified:
-            process.append((path, path, fid, kind))
+            if include_path(path):
+                process.append((path, path, fid, kind))
+        for path, fid, kind in delta.added:
+            if file == 'kind' and include_path(path):
+                process.append((path, path, fid, kind))
+        for path, fid, kind in delta.removed:
+            if file == 'kind' and include_path(path):
+                process.append((path, path, fid, kind))
+
+        process.sort(key=lambda x:x[1])
 
         for old_path, new_path, fid, kind in process:
-            old_lines = old_tree.get_file_lines(fid)
-            new_lines = new_tree.get_file_lines(fid)
+            try:
+                old_lines = old_tree.get_file_lines(fid)
+            except errors.NoSuchId:
+                old_lines = []
+            try:
+                new_lines = new_tree.get_file_lines(fid)
+            except errors.NoSuchId:
+                new_lines = []
             buffer = StringIO()
             if old_lines != new_lines:
                 try:
@@ -122,17 +147,13 @@
             else:
                 diff = ''
             out.append(util.Container(
-                          filename=rich_filename(new_path, kind),
-                          file_id=fid,
-                          chunks=self._process_diff(diff),
-                          raw_diff=diff))
+                filename=rich_filename(new_path, kind), file_id=fid,
+                chunks=self._process_diff(diff)))
 
         return out
 
-    def get_change_with_diff(self, revid, compare_revid):
+    def get_changes_with_diff(self, change, compare_revid, specific_path):
         h = self._history
-        change = h.get_changes([revid])[0]
-
         if compare_revid is None:
             if change.parents:
                 compare_revid = change.parents[0].revid
@@ -140,15 +161,13 @@
                 compare_revid = 'null:'
 
         rev_tree1 = h._branch.repository.revision_tree(compare_revid)
-        rev_tree2 = h._branch.repository.revision_tree(revid)
+        rev_tree2 = h._branch.repository.revision_tree(change.revid)
         delta = rev_tree2.changes_from(rev_tree1)
 
-        change.changes = h.parse_delta(delta)
-        change.changes.modified = self._parse_diffs(rev_tree1,
-                                                    rev_tree2,
-                                                    delta)
+        changes = h.parse_delta(delta)
 
-        return change
+        return changes, self._parse_diffs(
+            rev_tree1, rev_tree2, delta, specific_path)
 
     def get_values(self, path, kwargs, headers):
         h = self._history
@@ -177,16 +196,13 @@
             navigation.query = query
         util.fill_in_navigation(navigation)
 
-        change = self.get_change_with_diff(revid, compare_revid)
+        change = h.get_changes([revid])[0]
+        if path in ('', '/'):
+            path = None
+        change.changes, diffs = self.get_changes_with_diff(change, compare_revid, path)
         # add parent & merge-point branch-nick info, in case it's useful
         h.get_branch_nicks([change])
 
-        line_count_limit = DEFAULT_LINE_COUNT_LIMIT
-        line_count = 0
-        for file in change.changes.modified:
-            for chunk in file.chunks:
-                line_count += len(chunk.diff)
-
         # Directory Breadcrumbs
         directory_breadcrumbs = (
             util.directory_breadcrumbs(
@@ -198,6 +214,8 @@
             'branch': self._branch,
             'revid': revid,
             'change': change,
+            'diffs': diffs,
+            'specific_path': path,
             'start_revid': start_revid,
             'filter_file_id': filter_file_id,
             'util': util,
@@ -207,8 +225,5 @@
             'remember': remember,
             'compare_revid': compare_revid,
             'url': self._branch.context_url,
-            'line_count': line_count,
-            'line_count_limit': line_count_limit,
-            'show_plain_diffs': line_count > line_count_limit,
             'directory_breadcrumbs': directory_breadcrumbs,
         }

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2009-02-26 18:10:29 +0000
+++ loggerhead/history.py	2009-03-05 23:41:51 +0000
@@ -621,22 +621,23 @@
         removed = []
 
         for path, fid, kind in delta.added:
-            added.append((rich_filename(path, kind), fid))
+            added.append(util.Container(
+                filename=rich_filename(path, kind), file_id=fid, kind=kind))
 
         for path, fid, kind, text_modified, meta_modified in delta.modified:
-            modified.append(util.Container(filename=rich_filename(path, kind),
-                                           file_id=fid))
+            modified.append(util.Container(
+                filename=rich_filename(path, kind), file_id=fid))
 
         for old_path, new_path, fid, kind, text_modified, meta_modified in \
-delta.renamed:
-            renamed.append((rich_filename(old_path, kind),
-                            rich_filename(new_path, kind), fid))
-            if meta_modified or text_modified:
-                modified.append(util.Container(
-                    filename=rich_filename(new_path, kind), file_id=fid))
+                delta.renamed:
+            renamed.append(util.Container(
+                old_path=rich_filename(old_path, kind),
+                new_path=rich_filename(new_path, kind), file_id=fid,
+                text_modified=text_modified))
 
         for path, fid, kind in delta.removed:
-            removed.append((rich_filename(path, kind), fid))
+            removed.append(util.Container(
+                filename=rich_filename(path, kind), file_id=fid, kind=kind))
 
         return util.Container(added=added, renamed=renamed,
                               removed=removed, modified=modified)

=== modified file 'loggerhead/static/css/global.css'
--- loggerhead/static/css/global.css	2009-03-02 01:32:36 +0000
+++ loggerhead/static/css/global.css	2009-03-05 22:56:59 +0000
@@ -11,6 +11,9 @@
 	font-weight:normal;
 	outline:none;
 }
+b a:link, b a:visited {
+    font-weight: bold;
+}
 img {
 	padding:0px;
 	margin:0px;

=== modified file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	2009-02-18 10:36:48 +0000
+++ loggerhead/templatefunctions.py	2009-03-05 22:56:59 +0000
@@ -13,6 +13,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
+import cgi
 import os
 from loggerhead.zptsupport import zpt
 
@@ -37,18 +38,32 @@
 
 
 @templatefunc
-def file_change_summary(url, entry, modified_file_link):
+def file_change_summary(url, entry, link_style='normal', currently_showing=None):
+    if link_style == 'fragment':
+        def file_link(filename):
+            if currently_showing:
+                print filename, currently_showing, filename == currently_showing
+                if filename == currently_showing:
+                    return '<b><a href="#%s">%s</a></b>' % (
+                        cgi.escape(filename), cgi.escape(filename))
+                else:
+                    return revision_link(url, entry.revno, filename)
+            else:
+                return '<a href="#%s">%s</a>' % (
+                    cgi.escape(filename), cgi.escape(filename))
+    else:
+        file_link = lambda filename: revision_link(url, entry.revno, filename)
     return _pt('revisionfilechanges').expand(
-        url=url, entry=entry, modified_file_link=modified_file_link,
-        **templatefunctions)
+        url=url, entry=entry, file_link=file_link,
+        currently_showing=currently_showing, **templatefunctions)
 
 
 @templatefunc
-def revisioninfo(url, branch, entry, modified_file_link=None):
+def revisioninfo(url, branch, entry, include_file_list=False, currently_showing=None):
     from loggerhead import util
     return _pt('revisioninfo').expand(
         url=url, change=entry, branch=branch, util=util,
-        modified_file_link=modified_file_link,
+        include_file_list=include_file_list, currently_showing=currently_showing,
         **templatefunctions)
 
 
@@ -84,20 +99,6 @@
 
 
 @templatefunc
-def modified_file_link_rev(url, entry, item):
-    return _pt('modified-file-link-rev').expand(
-        url=url, entry=entry, item=item,
-        **templatefunctions)
-
-
-@templatefunc
-def modified_file_link_log(url, entry, item):
-    return _pt('modified-file-link-log').expand(
-        url=url, entry=entry, item=item,
-        **templatefunctions)
-
-
-@templatefunc
 def search_box(branch, navigation):
     return _pt('search-box').expand(branch=branch, navigation=navigation,
         **templatefunctions)
@@ -112,3 +113,15 @@
 def menu(branch, url, fileview_active=False):
     return _pt('menu').expand(branch=branch, url=url,
         fileview_active=fileview_active, **templatefunctions)
+
+
+@templatefunc
+def annotate_link(url, revno, path):
+    return '<a href="%s" title="Annotate %s">%s</a>'%(
+        url(['/annotate', revno, path]), cgi.escape(path), cgi.escape(path))
+
+@templatefunc
+def revision_link(url, revno, path):
+    return '<a href="%s" title="View changes to %s in revision %s">%s</a>'%(
+        url(['/revision', revno, path]), cgi.escape(path), cgi.escape(revno),
+        cgi.escape(path))

=== modified file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	2009-03-04 22:27:37 +0000
+++ loggerhead/templates/changelog.pt	2009-03-04 22:30:32 +0000
@@ -123,7 +123,7 @@
                        title="Show history" class="link"></a>
                   </li>
                   <li class="committerli" tal:content="python:util.hide_email(entry.author)"></li>
-                  <tal:block content="structure python:file_change_summary(url, entry, modified_file_link_log)" />
+                  <tal:block content="structure python:file_change_summary(url, entry)" />
                 </ul>
               </div>
             </td>

=== removed file 'loggerhead/templates/modified-file-link-rev.pt'
--- loggerhead/templates/modified-file-link-rev.pt	2008-06-19 21:39:15 +0000
+++ loggerhead/templates/modified-file-link-rev.pt	1970-01-01 00:00:00 +0000
@@ -1,7 +0,0 @@
-<tal:block>
-<li class="files"><a 
-     tal:attributes="href  string:#${item/filename}-s;
-                     title string:Jump to ${item/filename} below"
-     tal:content="item/filename" >
-</a></li>
-</tal:block>

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2009-02-26 21:01:19 +0000
+++ loggerhead/templates/revision.pt	2009-03-05 22:56:59 +0000
@@ -27,12 +27,31 @@
           <tal:no-link condition="not: branch/branch_link">
             <span metal:use-macro="breadcrumbs/directory"></span>
           </tal:no-link>
-          <span>: <tal:revno content="change/revno"></tal:revno>
-            <tal:compare-to condition="python:compare_revid is not None">
+          <span class="breadcrumb">:
+            <tal:revno condition="not:specific_path" content="change/revno"></tal:revno>
+            <a tal:condition="specific_path" tal:content="change/revno"
+               tal:attributes="href python:url(['/revision', change.revno])"
+               title="View changes to all files"></a>
+                      <tal:compare-to condition="python:compare_revid is not None">
               (compared to revision <tal:block content="python:history.get_revno(compare_revid)" />)
             </tal:compare-to>
-        </span></h1>
+          </span>
+          <span class="breadcrumb" tal:condition="specific_path">
+            : <tal:annotate content="structure python:annotate_link(url, change.revno, specific_path)" />
+          </span>
+        </h1>
         <tal:branch-info replace="structure python:branchinfo(branch)" />
+        <p tal:condition="not:specific_path">
+          Viewing all changes in revision <tal:revno content="change/revno" />.
+        </p>
+        <p tal:condition="specific_path">
+          <a tal:attributes="href python:url(['/revision', change.revno])">
+            &#xAB; back to all changes in this revision
+          </a>
+        </p>
+        <p tal:condition="specific_path">
+          Viewing changes to <tal:annotate content="structure python:annotate_link(url, change.revno, specific_path)" />
+        </p>
         <ul id="submenuTabs">
           <li id="first"><a tal:attributes="href python:url(['/files', change.revno]);
                                             title string:browse files at revision ${change/revno}"
@@ -72,7 +91,7 @@
           </ul>
         </tal:we-are-comparing>
 
-        <tal:revision-info replace="structure python:revisioninfo(url, branch, change, modified_file_link_rev)" />
+        <tal:revision-info replace="structure python:revisioninfo(url, branch, change, 'fragment', specific_path)" />
         <p class="expand" id="expand_all" style="display:none;"><a href="#">
             <img tal:attributes="src python:branch.static_url('/static/images/treeCollapsed.png')"
                  alt="expand all" /> expand all</a>
@@ -91,16 +110,16 @@
 
       <div metal:fill-slot="content">
         <div class="diff"
-             tal:repeat="item change/changes/modified">
+             tal:repeat="item diffs">
 
           <div class="diffBox">
             <img tal:attributes="src python:branch.static_url('/static/images/treeExpanded.png');
                                  title python:branch.static_url('/static/images/treeCollapsed.png');
                                  alt python:branch.static_url('/static/images/treeExpanded.png')"
                  class="expand_diff" />
-            <a tal:attributes="href python:url(['/annotate', change.revno], clear=1, file_id=item.file_id);
-                               id string:${item/filename}-s;
-                               title string:Annotate ${item/filename}"
+            <a tal:attributes="href python:url(['/revision', change.revno, item.filename], clear=1);
+                               id string:${item/filename};
+                               title string:View changes to ${item/filename} only"
                tal:content="item/filename">
             </a>
           </div>

=== modified file 'loggerhead/templates/revisionfilechanges.pt'
--- loggerhead/templates/revisionfilechanges.pt	2008-07-23 21:31:07 +0000
+++ loggerhead/templates/revisionfilechanges.pt	2009-03-05 23:41:51 +0000
@@ -1,58 +1,50 @@
 <tal:block>
-<ul tal:repeat="added entry/changes/added">
+  <ul tal:repeat="added entry/changes/added">
     <tal:block condition="repeat/added/start">
-        <li class="desc">files added:</li>
+      <li class="desc">files added:</li>
     </tal:block>
-    <li class="files"
-        tal:define="filename python:added[0];
-                    file_id  python:added[1]">
-        <a tal:attributes="href  python:url(['/annotate', entry.revno], file_id=file_id);
-                           title string:Annotate ${filename}"
-           tal:content="filename" class="link"></a>
+    <li class="files">
+      <tal:revision condition="python:added.kind == 'file'"
+                    content="structure python:file_link(added.filename)" />
+      <tal:revision condition="python:added.kind != 'file'"
+                    content="added/filename" />
     </li>
-</ul>
+  </ul>
 
-<ul tal:repeat="removed entry/changes/removed">
+  <ul tal:repeat="removed entry/changes/removed">
     <tal:block condition="repeat/removed/start">
-        <li class="desc">files removed:</li>
+      <li class="desc">files removed:</li>
     </tal:block>
-    <li class="files"
-        tal:define="filename python:removed[0];
-                    file_id  python:removed[1]">
-      <tal:block content="filename" />
+    <li class="files">
+      <tal:revision condition="python:removed.kind == 'file'"
+                    content="structure python:file_link(removed.filename)" />
+      <tal:revision condition="python:removed.kind != 'file'"
+                    content="removed/filename" />
     </li>
-</ul>
+  </ul>
 
-<ul tal:repeat="renamed entry/changes/renamed">
+  <ul tal:repeat="renamed entry/changes/renamed">
     <tal:block condition="repeat/renamed/start">
-        <li class="desc">files renamed:</li>
+      <li class="desc">files renamed:</li>
     </tal:block>
-    <li class="files"
-        tal:define="old_filename python:renamed[0];
-                    new_filename python:renamed[1];
-                    file_id      python:renamed[2]">
-      <a tal:attributes="href  python:url(['/annotate', entry.revno], file_id=file_id);
-                         title string:Annotate ${old_filename}"
-         tal:content="old_filename" class="filename">
+    <li class="files">
+      <tal:old-name content="renamed/old_filename" class="filename">
         old_filename
-      </a>
-      =>
-      <a tal:attributes="href  python:url(['/annotate', entry.revno], file_id=file_id);
-                         title string:Annotate ${new_filename}"
-         tal:content="new_filename">
+      </tal:old-name>
+      =&gt;
+      <tal:revision condition="renamed/text_modified" content="structure python:file_link(renamed.new_filename)" />
+      <tal:old-name condition="not:renamed/text_modified" content="renamed/new_filename" class="filename">
         new_filename
-      </a>
+      </tal:old-name>
     </li>
-</ul>
+  </ul>
 
-<ul tal:repeat="item entry/changes/modified">
+  <ul tal:repeat="item entry/changes/modified">
     <tal:block condition="repeat/item/start">
-        <li class="desc">files modified:</li>
+      <li class="desc">files modified:</li>
     </tal:block>
     <li class="files">
-      <a tal:attributes="href  python:url(['/annotate', entry.revno], file_id=item.file_id);
-                         title string:Annotate ${item/filename}"
-                         tal:content="item/filename"/>
+      <tal:revision content="structure python:file_link(item.filename)" />
     </li>
-</ul>
+  </ul>
 </tal:block>

=== modified file 'loggerhead/templates/revisioninfo.pt'
--- loggerhead/templates/revisioninfo.pt	2009-03-03 21:39:54 +0000
+++ loggerhead/templates/revisioninfo.pt	2009-03-05 22:56:59 +0000
@@ -15,9 +15,8 @@
         <div class="revid" tal:content="string:Revision ID: ${change/revid}"></div>
         <div class="information" tal:content="structure python:util.fixed_width(change.comment)"></div>
     </div>
-    <ul id="list-files">
-        <tal:block condition="nocall:modified_file_link">
-        <tal:block content="structure python:file_change_summary(url, change, modified_file_link)" /></tal:block>
+    <ul id="list-files" tal:condition="include_file_list">
+      <tal:block content="structure python:file_change_summary(url, change, 'fragment', currently_showing)" />
     </ul>
     <div class="clear"><!-- --></div>
 </div>

=== modified file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	2009-02-23 20:32:43 +0000
+++ loggerhead/tests/test_controllers.py	2009-02-26 06:31:24 +0000
@@ -32,8 +32,8 @@
         self.build_tree_contents(shape1)
         tree.smart_add([])
         tree.commit('')
-        tree.smart_add([])
         self.build_tree_contents(shape2)
+        tree.smart_add([])
         tree.commit('')
         tree.branch.lock_read()
         self.addCleanup(tree.branch.unlock)
@@ -56,6 +56,22 @@
             rev_ui.get_values('2', {}, []),
             dict)
 
+    def test_get_changes_with_diff(self):
+        branch, rev_ui = self.make_bzrbranch_and_revision_ui_for_tree_shapes(
+            [('file', 'oldcontents'), ('file2', 'oldcontents')],
+            [('file', 'newcontents'), ('file2', 'oldcontents')])
+        change = rev_ui._history.get_changes([branch.last_revision()])[0]
+        changes, diffs = rev_ui.get_changes_with_diff(change, None, None)
+        self.assertEqual(1, len(diffs))
+
+    def test_get_changes_with_diff_specific_path(self):
+        branch, rev_ui = self.make_bzrbranch_and_revision_ui_for_tree_shapes(
+            [('file', 'oldcontents'), ('file2', 'oldcontents')],
+            [('file', 'newcontents'), ('file2', 'newcontents')])
+        change = rev_ui._history.get_changes([branch.last_revision()])[0]
+        changes, diffs = rev_ui.get_changes_with_diff(change, None, 'file')
+        self.assertEqual(1, len(diffs))
+
 class TestAnnotateUI(BasicTests):
 
     def make_annotate_ui_for_file_history(self, file_id, rev_ids_texts):

